/*------------------------------------------------------------------------------
* correlator.c: mex function correlator
*
* 2010/11/03 0.1 new
*-----------------------------------------------------------------------------*/
#include "mex.h"
#include "sdr.h"

/* mex interface -------------------------------------------------------------*/
extern void mexFunction(int nargout, mxArray *argout[], int nargin,
                        const mxArray *argin[])
{
    double ti,freq,phi0,crate,coff,*I,*Q;
    char *data;
    int dtype,ctype,sat,s,m,n;
    
    if (nargin<11||
        mxGetM(argin[0])<1||mxGetM(argin[1])<1||mxGetM(argin[2])<1||
        mxGetM(argin[3])<1||mxGetM(argin[4])<1||mxGetM(argin[5])<1||
        mxGetM(argin[6])<1||mxGetM(argin[7])<1||mxGetM(argin[8])<1||
        mxGetM(argin[9])<1||mxGetM(argin[10])<1||!mxIsInt8(argin[0])) {
        mexErrMsgTxt("argin error");
    }
    data =(char *)mxGetPr(argin[ 0]); /* I: sampling signal */
    dtype=(int)  *mxGetPr(argin[ 1]); /* I: sampling type (0:real,1:complex) */
    ti   =       *mxGetPr(argin[ 2]); /* I: sampling interval (s) */
    freq =       *mxGetPr(argin[ 3]); /* I: frequency (Hz) */
    phi0 =       *mxGetPr(argin[ 4]); /* I: phase offset (rad) */
    sat  =(int)  *mxGetPr(argin[ 5]); /* I: satellite number */
    ctype=(int)  *mxGetPr(argin[ 6]); /* I: code type */
    crate=       *mxGetPr(argin[ 7]); /* I: code chip rate (chip/s) */
    coff =       *mxGetPr(argin[ 8]); /* I: code offset (chip) */
    s    =(int)  *mxGetPr(argin[ 9]); /* I: correlator spacing (sample) */
    m    =(int)  *mxGetPr(argin[10]); /* I: number of side correlators */
    
    n=(int)mxGetN(argin[0]);
    if (dtype) n/=2;
    
    argout[0]=mxCreateDoubleMatrix(1,1+2*m,mxREAL);
    argout[1]=mxCreateDoubleMatrix(1,1+2*m,mxREAL);
    
    /* I=(I_P,I_E1,I_L1,I_E2,I_L2,...), Q=(Q_P,...) */
    I    =       mxGetPr(argout[0]); /* O: correlator results I */
    Q    =       mxGetPr(argout[1]); /* O: correlator results Q */
    
    correlator(data,dtype,ti,n,freq,phi0,sat,ctype,crate,coff,s,m,I,Q);
}
